@@ -1,165 +1,180 @@
 """Amazon S3 Read Module (PRIVATE)."""
 
+from __future__ import annotations
+
 import datetime
+import itertools
 import logging
 import pprint
-from typing import Any, Callable, Dict, Iterator, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Callable, Iterator
 
 import boto3
 import pandas as pd
-import pandas.io.parsers
-from pandas.io.common import infer_compression
+from typing_extensions import Literal
 
 from awswrangler import _utils, exceptions
-from awswrangler.s3._fs import open_s3_object
+from awswrangler._distributed import engine
+from awswrangler._executor import _BaseExecutor, _get_executor
 from awswrangler.s3._list import _path2list
 from awswrangler.s3._read import (
     _apply_partition_filter,
-    _apply_partitions,
+    _check_version_id,
     _get_path_ignore_suffix,
     _get_path_root,
     _union,
 )
+from awswrangler.s3._read_text_core import _read_text_file, _read_text_files_chunked
+from awswrangler.typing import RaySettings
+
+if TYPE_CHECKING:
+    from mypy_boto3_s3 import S3Client
 
 _logger: logging.Logger = logging.getLogger(__name__)
 
 
-def _get_read_details(path: str, pandas_kwargs: Dict[str, Any]) -> Tuple[str, Optional[str], Optional[str]]:
-    if pandas_kwargs.get("compression", "infer") == "infer":
-        pandas_kwargs["compression"] = infer_compression(path, compression="infer")
-    mode: str = "r" if pandas_kwargs.get("compression") is None else "rb"
-    encoding: Optional[str] = pandas_kwargs.get("encoding", None)
-    newline: Optional[str] = pandas_kwargs.get("lineterminator", None)
-    return mode, encoding, newline
-
-
-def _read_text_chunked(
-    paths: List[str],
-    chunksize: int,
-    parser_func: Callable[..., pd.DataFrame],
-    path_root: Optional[str],
-    boto3_session: boto3.Session,
-    pandas_kwargs: Dict[str, Any],
-    s3_additional_kwargs: Optional[Dict[str, str]],
-    dataset: bool,
-    use_threads: bool,
-) -> Iterator[pd.DataFrame]:
-    for path in paths:
-        _logger.debug("path: %s", path)
-        mode, encoding, newline = _get_read_details(path=path, pandas_kwargs=pandas_kwargs)
-        with open_s3_object(
-            path=path,
-            mode=mode,
-            s3_block_size=10_485_760,  # 10 MB (10 * 2**20)
-            encoding=encoding,
-            use_threads=use_threads,
-            s3_additional_kwargs=s3_additional_kwargs,
-            newline=newline,
-            boto3_session=boto3_session,
-        ) as f:
-            reader: pandas.io.parsers.TextFileReader = parser_func(f, chunksize=chunksize, **pandas_kwargs)
-            for df in reader:
-                yield _apply_partitions(df=df, dataset=dataset, path=path, path_root=path_root)
-
-
-def _read_text_file(
-    path: str,
-    parser_func: Callable[..., pd.DataFrame],
-    path_root: Optional[str],
-    boto3_session: Union[boto3.Session, Dict[str, Optional[str]]],
-    pandas_kwargs: Dict[str, Any],
-    s3_additional_kwargs: Optional[Dict[str, str]],
-    dataset: bool,
-    use_threads: bool,
-) -> pd.DataFrame:
-    mode, encoding, newline = _get_read_details(path=path, pandas_kwargs=pandas_kwargs)
-    with open_s3_object(
-        path=path,
-        mode=mode,
-        use_threads=use_threads,
-        s3_block_size=-1,  # One shot download
-        encoding=encoding,
-        s3_additional_kwargs=s3_additional_kwargs,
-        newline=newline,
-        boto3_session=boto3_session,
-    ) as f:
-        df: pd.DataFrame = parser_func(f, **pandas_kwargs)
-    return _apply_partitions(df=df, dataset=dataset, path=path, path_root=path_root)
+def _resolve_format(read_format: str) -> Any:
+    if read_format == "csv":
+        return pd.read_csv
+    if read_format == "fwf":
+        return pd.read_fwf
+    if read_format == "json":
+        return pd.read_json
+    raise exceptions.UnsupportedType("Unsupported read format")
 
 
+@engine.dispatch_on_engine
 def _read_text(
-    parser_func: Callable[..., pd.DataFrame],
-    path: Union[str, List[str]],
-    path_suffix: Union[str, List[str], None],
-    path_ignore_suffix: Union[str, List[str], None],
-    use_threads: bool,
-    last_modified_begin: Optional[datetime.datetime],
-    last_modified_end: Optional[datetime.datetime],
-    boto3_session: Optional[boto3.Session],
-    s3_additional_kwargs: Optional[Dict[str, str]],
-    chunksize: Optional[int],
+    read_format: str,
+    paths: list[str],
+    path_root: str | None,
+    use_threads: bool | int,
+    s3_client: "S3Client",
+    s3_additional_kwargs: dict[str, str] | None,
     dataset: bool,
-    partition_filter: Optional[Callable[[Dict[str, str]], bool]],
     ignore_index: bool,
+    parallelism: int,
+    version_ids: dict[str, str] | None,
+    pandas_kwargs: dict[str, Any],
+) -> pd.DataFrame:
+    parser_func = _resolve_format(read_format)
+    executor: _BaseExecutor = _get_executor(use_threads=use_threads)
+    tables = executor.map(
+        _read_text_file,
+        s3_client,
+        paths,
+        [version_ids.get(p) if isinstance(version_ids, dict) else None for p in paths],
+        itertools.repeat(parser_func),
+        itertools.repeat(path_root),
+        itertools.repeat(pandas_kwargs),
+        itertools.repeat(s3_additional_kwargs),
+        itertools.repeat(dataset),
+    )
+    return _union(dfs=tables, ignore_index=ignore_index)
+
+
+def _read_text_format(
+    read_format: str,
+    path: str | list[str],
+    path_suffix: str | list[str] | None,
+    path_ignore_suffix: str | list[str] | None,
+    ignore_empty: bool,
+    use_threads: bool | int,
+    last_modified_begin: datetime.datetime | None,
+    last_modified_end: datetime.datetime | None,
+    s3_client: "S3Client",
+    s3_additional_kwargs: dict[str, str] | None,
+    chunksize: int | None,
+    dataset: bool,
+    partition_filter: Callable[[dict[str, str]], bool] | None,
+    ignore_index: bool,
+    ray_args: RaySettings | None,
+    version_id: str | dict[str, str] | None = None,
     **pandas_kwargs: Any,
-) -> Union[pd.DataFrame, Iterator[pd.DataFrame]]:
+) -> pd.DataFrame | Iterator[pd.DataFrame]:
     if "iterator" in pandas_kwargs:
         raise exceptions.InvalidArgument("Please, use the chunksize argument instead of iterator.")
-    session: boto3.Session = _utils.ensure_session(session=boto3_session)
-    paths: List[str] = _path2list(
+    paths: list[str] = _path2list(
         path=path,
-        boto3_session=session,
+        s3_client=s3_client,
         suffix=path_suffix,
         ignore_suffix=_get_path_ignore_suffix(path_ignore_suffix=path_ignore_suffix),
+        ignore_empty=ignore_empty,
         last_modified_begin=last_modified_begin,
         last_modified_end=last_modified_end,
+        s3_additional_kwargs=s3_additional_kwargs,
     )
-    path_root: Optional[str] = _get_path_root(path=path, dataset=dataset)
+
+    path_root: str | None = _get_path_root(path=path, dataset=dataset)
     if path_root is not None:
         paths = _apply_partition_filter(path_root=path_root, paths=paths, filter_func=partition_filter)
     if len(paths) < 1:
         raise exceptions.NoFilesFound(f"No files Found on: {path}.")
-    _logger.debug("paths:\n%s", paths)
 
-    args: Dict[str, Any] = {
-        "parser_func": parser_func,
-        "boto3_session": session,
+    version_ids = _check_version_id(paths=paths, version_id=version_id)
+
+    args: dict[str, Any] = {
+        "parser_func": _resolve_format(read_format),
+        "s3_client": s3_client,
         "dataset": dataset,
         "path_root": path_root,
         "pandas_kwargs": pandas_kwargs,
         "s3_additional_kwargs": s3_additional_kwargs,
         "use_threads": use_threads,
     }
-    _logger.debug("args:\n%s", pprint.pformat(args))
-    ret: Union[pd.DataFrame, Iterator[pd.DataFrame]]
+    _logger.debug("Read args:\n%s", pprint.pformat(args))
+
     if chunksize is not None:
-        ret = _read_text_chunked(paths=paths, chunksize=chunksize, **args)
-    elif len(paths) == 1:
-        ret = _read_text_file(path=paths[0], **args)
-    else:
-        ret = _union(dfs=[_read_text_file(path=p, **args) for p in paths], ignore_index=ignore_index)
-    return ret
+        return _read_text_files_chunked(
+            paths=paths,
+            version_ids=version_ids,
+            chunksize=chunksize,
+            **args,
+        )
+
+    ray_args = ray_args if ray_args else {}
+    return _read_text(
+        read_format,
+        paths=paths,
+        path_root=path_root,
+        use_threads=use_threads,
+        s3_client=s3_client,
+        s3_additional_kwargs=s3_additional_kwargs,
+        dataset=dataset,
+        ignore_index=ignore_index,
+        parallelism=ray_args.get("parallelism", -1),
+        version_ids=version_ids,
+        pandas_kwargs=pandas_kwargs,
+    )
 
 
+@_utils.validate_distributed_kwargs(
+    unsupported_kwargs=["boto3_session"],
+)
 def read_csv(
-    path: Union[str, List[str]],
-    path_suffix: Union[str, List[str], None] = None,
-    path_ignore_suffix: Union[str, List[str], None] = None,
-    use_threads: bool = True,
-    last_modified_begin: Optional[datetime.datetime] = None,
-    last_modified_end: Optional[datetime.datetime] = None,
-    boto3_session: Optional[boto3.Session] = None,
-    s3_additional_kwargs: Optional[Dict[str, Any]] = None,
-    chunksize: Optional[int] = None,
+    path: str | list[str],
+    path_suffix: str | list[str] | None = None,
+    path_ignore_suffix: str | list[str] | None = None,
+    version_id: str | dict[str, str] | None = None,
+    ignore_empty: bool = True,
+    use_threads: bool | int = True,
+    last_modified_begin: datetime.datetime | None = None,
+    last_modified_end: datetime.datetime | None = None,
+    boto3_session: boto3.Session | None = None,
+    s3_additional_kwargs: dict[str, Any] | None = None,
+    dtype_backend: Literal["numpy_nullable", "pyarrow"] = "numpy_nullable",
+    chunksize: int | None = None,
     dataset: bool = False,
-    partition_filter: Optional[Callable[[Dict[str, str]], bool]] = None,
+    partition_filter: Callable[[dict[str, str]], bool] | None = None,
+    ray_args: RaySettings | None = None,
     **pandas_kwargs: Any,
-) -> Union[pd.DataFrame, Iterator[pd.DataFrame]]:
-    """Read CSV file(s) from from a received S3 prefix or list of S3 objects paths.
+) -> pd.DataFrame | Iterator[pd.DataFrame]:
+    """Read CSV file(s) from a received S3 prefix or list of S3 objects paths.
 
     This function accepts Unix shell-style wildcards in the path argument.
     * (matches everything), ? (matches any single character),
     [seq] (matches any character in seq), [!seq] (matches any character not in seq).
+    If you want to use a path which includes Unix shell-style wildcard characters (`*, ?, []`),
+    you can use `glob.escape(path)` before passing the path to this function.
 
     Note
     ----
@@ -180,12 +195,20 @@ def read_csv(
         S3 prefix (accepts Unix shell-style wildcards)
         (e.g. s3://bucket/prefix) or list of S3 objects paths (e.g. ``[s3://bucket/key0, s3://bucket/key1]``).
     path_suffix: Union[str, List[str], None]
-        Suffix or List of suffixes for filtering S3 keys.
+        Suffix or List of suffixes to be read (e.g. [".csv"]).
+        If None, will try to read all files. (default)
     path_ignore_suffix: Union[str, List[str], None]
-        Suffix or List of suffixes for S3 keys to be ignored.
-    use_threads : bool
+        Suffix or List of suffixes for S3 keys to be ignored.(e.g. ["_SUCCESS"]).
+        If None, will try to read all files. (default)
+    version_id: Optional[Union[str, Dict[str, str]]]
+        Version id of the object or mapping of object path to version id.
+        (e.g. {'s3://bucket/key0': '121212', 's3://bucket/key1': '343434'})
+    ignore_empty: bool
+        Ignore files with 0 bytes.
+    use_threads : Union[bool, int]
         True to enable concurrent requests, False to disable multiple threads.
         If enabled os.cpu_count() will be used as the max number of threads.
+        If integer is provided, specified number is used.
     last_modified_begin
         Filter the s3 files by the Last modified date of the object.
         The filter is applied only after list all s3 files.
@@ -194,8 +217,14 @@ def read_csv(
         The filter is applied only after list all s3 files.
     boto3_session : boto3.Session(), optional
         Boto3 Session. The default boto3 session will be used if boto3_session receive None.
-    s3_additional_kwargs : Optional[Dict[str, Any]]
+    pyarrow_additional_kwargs: dict[str, Any], optional
         Forward to botocore requests, only "SSECustomerAlgorithm" and "SSECustomerKey" arguments will be considered.
+    dtype_backend: str, optional
+        Which dtype_backend to use, e.g. whether a DataFrame should have NumPy arrays,
+        nullable dtypes are used for all dtypes that have a nullable implementation when
+        “numpy_nullable” is set, pyarrow is used for all dtypes if “pyarrow” is set.
+
+        The dtype_backends are still experimential. The "pyarrow" backend is only supported with Pandas 2.0 or above.
     chunksize: int, optional
         If specified, return an generator where chunksize is the number of rows to include in each chunk.
     dataset : bool
@@ -207,10 +236,14 @@ def read_csv(
         This function MUST return a bool, True to read the partition or False to ignore it.
         Ignored if `dataset=False`.
         E.g ``lambda x: True if x["year"] == "2020" and x["month"] == "1" else False``
-        https://github.com/awslabs/aws-data-wrangler/blob/master/tutorials/023%20-%20Flexible%20Partitions%20Filter.ipynb
+        https://aws-sdk-pandas.readthedocs.io/en/3.5.2/tutorials/023%20-%20Flexible%20Partitions%20Filter.html
+    s3_additional_kwargs: dict[str, Any], optional
+        Forwarded to botocore requests.
+    ray_args: typing.RaySettings, optional
+        Parameters of the Ray Modin settings. Only used when distributed computing is used with Ray and Modin installed.
     pandas_kwargs :
-        KEYWORD arguments forwarded to pandas.read_csv(). You can NOT pass `pandas_kwargs` explicit, just add valid
-        Pandas arguments in the function call and Wrangler will accept it.
+        KEYWORD arguments forwarded to pandas.read_csv(). You can NOT pass `pandas_kwargs` explicitly, just add valid
+        Pandas arguments in the function call and awswrangler will accept it.
         e.g. wr.s3.read_csv('s3://bucket/prefix/', sep='|', na_values=['null', 'none'], skip_blank_lines=True)
         https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_csv.html
 
@@ -252,18 +285,25 @@ def read_csv(
     """
     if "pandas_kwargs" in pandas_kwargs:
         raise exceptions.InvalidArgument(
-            "You can NOT pass `pandas_kwargs` explicit, just add valid "
-            "Pandas arguments in the function call and Wrangler will accept it."
+            "You can NOT pass `pandas_kwargs` explicitly, just add valid "
+            "Pandas arguments in the function call and awswrangler will accept it."
             "e.g. wr.s3.read_csv('s3://bucket/prefix/', sep='|', skip_blank_lines=True)"
         )
+
+    if dtype_backend != "numpy_nullable":
+        pandas_kwargs["dtype_backend"] = dtype_backend
+
+    s3_client = _utils.client(service_name="s3", session=boto3_session)
     ignore_index: bool = "index_col" not in pandas_kwargs
-    return _read_text(
-        parser_func=pd.read_csv,
+    return _read_text_format(
+        read_format="csv",
         path=path,
         path_suffix=path_suffix,
         path_ignore_suffix=path_ignore_suffix,
+        version_id=version_id,
+        ignore_empty=ignore_empty,
         use_threads=use_threads,
-        boto3_session=boto3_session,
+        s3_client=s3_client,
         s3_additional_kwargs=s3_additional_kwargs,
         chunksize=chunksize,
         dataset=dataset,
@@ -271,29 +311,38 @@ def read_csv(
         last_modified_begin=last_modified_begin,
         last_modified_end=last_modified_end,
         ignore_index=ignore_index,
+        ray_args=ray_args,
         **pandas_kwargs,
     )
 
 
+@_utils.validate_distributed_kwargs(
+    unsupported_kwargs=["boto3_session"],
+)
 def read_fwf(
-    path: Union[str, List[str]],
-    path_suffix: Union[str, List[str], None] = None,
-    path_ignore_suffix: Union[str, List[str], None] = None,
-    use_threads: bool = True,
-    last_modified_begin: Optional[datetime.datetime] = None,
-    last_modified_end: Optional[datetime.datetime] = None,
-    boto3_session: Optional[boto3.Session] = None,
-    s3_additional_kwargs: Optional[Dict[str, Any]] = None,
-    chunksize: Optional[int] = None,
+    path: str | list[str],
+    path_suffix: str | list[str] | None = None,
+    path_ignore_suffix: str | list[str] | None = None,
+    version_id: str | dict[str, str] | None = None,
+    ignore_empty: bool = True,
+    use_threads: bool | int = True,
+    last_modified_begin: datetime.datetime | None = None,
+    last_modified_end: datetime.datetime | None = None,
+    boto3_session: boto3.Session | None = None,
+    s3_additional_kwargs: dict[str, Any] | None = None,
+    chunksize: int | None = None,
     dataset: bool = False,
-    partition_filter: Optional[Callable[[Dict[str, str]], bool]] = None,
+    partition_filter: Callable[[dict[str, str]], bool] | None = None,
+    ray_args: RaySettings | None = None,
     **pandas_kwargs: Any,
-) -> Union[pd.DataFrame, Iterator[pd.DataFrame]]:
-    """Read fixed-width formatted file(s) from from a received S3 prefix or list of S3 objects paths.
+) -> pd.DataFrame | Iterator[pd.DataFrame]:
+    """Read fixed-width formatted file(s) from a received S3 prefix or list of S3 objects paths.
 
     This function accepts Unix shell-style wildcards in the path argument.
     * (matches everything), ? (matches any single character),
     [seq] (matches any character in seq), [!seq] (matches any character not in seq).
+    If you want to use a path which includes Unix shell-style wildcard characters (`*, ?, []`),
+    you can use `glob.escape(path)` before passing the path to this function.
 
     Note
     ----
@@ -314,12 +363,20 @@ def read_fwf(
         S3 prefix (accepts Unix shell-style wildcards)
         (e.g. s3://bucket/prefix) or list of S3 objects paths (e.g. ``[s3://bucket/key0, s3://bucket/key1]``).
     path_suffix: Union[str, List[str], None]
-        Suffix or List of suffixes for filtering S3 keys.
+        Suffix or List of suffixes to be read (e.g. [".txt"]).
+        If None, will try to read all files. (default)
     path_ignore_suffix: Union[str, List[str], None]
-        Suffix or List of suffixes for S3 keys to be ignored.
-    use_threads : bool
+        Suffix or List of suffixes for S3 keys to be ignored.(e.g. ["_SUCCESS"]).
+        If None, will try to read all files. (default)
+    version_id: Optional[Union[str, Dict[str, str]]]
+        Version id of the object or mapping of object path to version id.
+        (e.g. {'s3://bucket/key0': '121212', 's3://bucket/key1': '343434'})
+    ignore_empty: bool
+        Ignore files with 0 bytes.
+    use_threads : Union[bool, int]
         True to enable concurrent requests, False to disable multiple threads.
         If enabled os.cpu_count() will be used as the max number of threads.
+        If integer is provided, specified number is used.
     last_modified_begin
         Filter the s3 files by the Last modified date of the object.
         The filter is applied only after list all s3 files.
@@ -328,7 +385,7 @@ def read_fwf(
         The filter is applied only after list all s3 files.
     boto3_session : boto3.Session(), optional
         Boto3 Session. The default boto3 session will be used if boto3_session receive None.
-    s3_additional_kwargs : Optional[Dict[str, Any]]
+    pyarrow_additional_kwargs: dict[str, Any], optional
         Forward to botocore requests, only "SSECustomerAlgorithm" and "SSECustomerKey" arguments will be considered.
     chunksize: int, optional
         If specified, return an generator where chunksize is the number of rows to include in each chunk.
@@ -341,10 +398,14 @@ def read_fwf(
         This function MUST return a bool, True to read the partition or False to ignore it.
         Ignored if `dataset=False`.
         E.g ``lambda x: True if x["year"] == "2020" and x["month"] == "1" else False``
-        https://github.com/awslabs/aws-data-wrangler/blob/master/tutorials/023%20-%20Flexible%20Partitions%20Filter.ipynb
+        https://aws-sdk-pandas.readthedocs.io/en/3.5.2/tutorials/023%20-%20Flexible%20Partitions%20Filter.html
+    s3_additional_kwargs: dict[str, Any], optional
+        Forwarded to botocore requests.
+    ray_args: typing.RaySettings, optional
+        Parameters of the Ray Modin settings. Only used when distributed computing is used with Ray and Modin installed.
     pandas_kwargs:
         KEYWORD arguments forwarded to pandas.read_fwf(). You can NOT pass `pandas_kwargs` explicit, just add valid
-        Pandas arguments in the function call and Wrangler will accept it.
+        Pandas arguments in the function call and awswrangler will accept it.
         e.g. wr.s3.read_fwf(path='s3://bucket/prefix/', widths=[1, 3], names=["c0", "c1"])
         https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_fwf.html
 
@@ -387,16 +448,19 @@ def read_fwf(
     if "pandas_kwargs" in pandas_kwargs:
         raise exceptions.InvalidArgument(
             "You can NOT pass `pandas_kwargs` explicit, just add valid "
-            "Pandas arguments in the function call and Wrangler will accept it."
+            "Pandas arguments in the function call and awswrangler will accept it."
             "e.g. wr.s3.read_fwf(path, widths=[1, 3], names=['c0', 'c1'])"
         )
-    return _read_text(
-        parser_func=pd.read_fwf,
+    s3_client = _utils.client(service_name="s3", session=boto3_session)
+    return _read_text_format(
+        read_format="fwf",
         path=path,
         path_suffix=path_suffix,
         path_ignore_suffix=path_ignore_suffix,
+        version_id=version_id,
+        ignore_empty=ignore_empty,
         use_threads=use_threads,
-        boto3_session=boto3_session,
+        s3_client=s3_client,
         s3_additional_kwargs=s3_additional_kwargs,
         chunksize=chunksize,
         dataset=dataset,
@@ -405,30 +469,40 @@ def read_fwf(
         last_modified_end=last_modified_end,
         ignore_index=True,
         sort_index=False,
+        ray_args=ray_args,
         **pandas_kwargs,
     )
 
 
+@_utils.validate_distributed_kwargs(
+    unsupported_kwargs=["boto3_session"],
+)
 def read_json(
-    path: Union[str, List[str]],
-    path_suffix: Union[str, List[str], None] = None,
-    path_ignore_suffix: Union[str, List[str], None] = None,
+    path: str | list[str],
+    path_suffix: str | list[str] | None = None,
+    path_ignore_suffix: str | list[str] | None = None,
+    version_id: str | dict[str, str] | None = None,
+    ignore_empty: bool = True,
     orient: str = "columns",
-    use_threads: bool = True,
-    last_modified_begin: Optional[datetime.datetime] = None,
-    last_modified_end: Optional[datetime.datetime] = None,
-    boto3_session: Optional[boto3.Session] = None,
-    s3_additional_kwargs: Optional[Dict[str, Any]] = None,
-    chunksize: Optional[int] = None,
+    use_threads: bool | int = True,
+    last_modified_begin: datetime.datetime | None = None,
+    last_modified_end: datetime.datetime | None = None,
+    boto3_session: boto3.Session | None = None,
+    s3_additional_kwargs: dict[str, Any] | None = None,
+    dtype_backend: Literal["numpy_nullable", "pyarrow"] = "numpy_nullable",
+    chunksize: int | None = None,
     dataset: bool = False,
-    partition_filter: Optional[Callable[[Dict[str, str]], bool]] = None,
+    partition_filter: Callable[[dict[str, str]], bool] | None = None,
+    ray_args: RaySettings | None = None,
     **pandas_kwargs: Any,
-) -> Union[pd.DataFrame, Iterator[pd.DataFrame]]:
-    """Read JSON file(s) from from a received S3 prefix or list of S3 objects paths.
+) -> pd.DataFrame | Iterator[pd.DataFrame]:
+    """Read JSON file(s) from a received S3 prefix or list of S3 objects paths.
 
     This function accepts Unix shell-style wildcards in the path argument.
     * (matches everything), ? (matches any single character),
     [seq] (matches any character in seq), [!seq] (matches any character not in seq).
+    If you want to use a path which includes Unix shell-style wildcard characters (`*, ?, []`),
+    you can use `glob.escape(path)` before passing the path to this function.
 
     Note
     ----
@@ -449,14 +523,22 @@ def read_json(
         S3 prefix (accepts Unix shell-style wildcards)
         (e.g. s3://bucket/prefix) or list of S3 objects paths (e.g. ``[s3://bucket/key0, s3://bucket/key1]``).
     path_suffix: Union[str, List[str], None]
-        Suffix or List of suffixes for filtering S3 keys.
+        Suffix or List of suffixes to be read (e.g. [".json"]).
+        If None, will try to read all files. (default)
     path_ignore_suffix: Union[str, List[str], None]
-        Suffix or List of suffixes for S3 keys to be ignored.
+        Suffix or List of suffixes for S3 keys to be ignored.(e.g. ["_SUCCESS"]).
+        If None, will try to read all files. (default)
+    version_id: Optional[Union[str, Dict[str, str]]]
+        Version id of the object or mapping of object path to version id.
+        (e.g. {'s3://bucket/key0': '121212', 's3://bucket/key1': '343434'})
+    ignore_empty: bool
+        Ignore files with 0 bytes.
     orient : str
         Same as Pandas: https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_json.html
-    use_threads : bool
+    use_threads : Union[bool, int]
         True to enable concurrent requests, False to disable multiple threads.
         If enabled os.cpu_count() will be used as the max number of threads.
+        If integer is provided, specified number is used.
     last_modified_begin
         Filter the s3 files by the Last modified date of the object.
         The filter is applied only after list all s3 files.
@@ -465,8 +547,14 @@ def read_json(
         The filter is applied only after list all s3 files.
     boto3_session : boto3.Session(), optional
         Boto3 Session. The default boto3 session will be used if boto3_session receive None.
-    s3_additional_kwargs : Optional[Dict[str, Any]]
+    pyarrow_additional_kwargs: dict[str, Any], optional
         Forward to botocore requests, only "SSECustomerAlgorithm" and "SSECustomerKey" arguments will be considered.
+    dtype_backend: str, optional
+        Which dtype_backend to use, e.g. whether a DataFrame should have NumPy arrays,
+        nullable dtypes are used for all dtypes that have a nullable implementation when
+        “numpy_nullable” is set, pyarrow is used for all dtypes if “pyarrow” is set.
+
+        The dtype_backends are still experimential. The "pyarrow" backend is only supported with Pandas 2.0 or above.
     chunksize: int, optional
         If specified, return an generator where chunksize is the number of rows to include in each chunk.
     dataset: bool
@@ -479,10 +567,14 @@ def read_json(
         This function MUST return a bool, True to read the partition or False to ignore it.
         Ignored if `dataset=False`.
         E.g ``lambda x: True if x["year"] == "2020" and x["month"] == "1" else False``
-        https://github.com/awslabs/aws-data-wrangler/blob/master/tutorials/023%20-%20Flexible%20Partitions%20Filter.ipynb
+        https://aws-sdk-pandas.readthedocs.io/en/3.5.2/tutorials/023%20-%20Flexible%20Partitions%20Filter.html
+    s3_additional_kwargs: dict[str, Any], optional
+        Forwarded to botocore requests.
+    ray_args: typing.RaySettings, optional
+        Parameters of the Ray Modin settings. Only used when distributed computing is used with Ray and Modin installed.
     pandas_kwargs:
         KEYWORD arguments forwarded to pandas.read_json(). You can NOT pass `pandas_kwargs` explicit, just add valid
-        Pandas arguments in the function call and Wrangler will accept it.
+        Pandas arguments in the function call and awswrangler will accept it.
         e.g. wr.s3.read_json('s3://bucket/prefix/', lines=True, keep_default_dates=True)
         https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.read_json.html
 
@@ -525,20 +617,28 @@ def read_json(
     if "pandas_kwargs" in pandas_kwargs:
         raise exceptions.InvalidArgument(
             "You can NOT pass `pandas_kwargs` explicit, just add valid "
-            "Pandas arguments in the function call and Wrangler will accept it."
+            "Pandas arguments in the function call and awswrangler will accept it."
             "e.g. wr.s3.read_json(path, lines=True, keep_default_dates=True)"
         )
+    if dtype_backend != "numpy_nullable":
+        pandas_kwargs["dtype_backend"] = dtype_backend
+
+    s3_client = _utils.client(service_name="s3", session=boto3_session)
+
     if (dataset is True) and ("lines" not in pandas_kwargs):
         pandas_kwargs["lines"] = True
     pandas_kwargs["orient"] = orient
     ignore_index: bool = orient not in ("split", "index", "columns")
-    return _read_text(
-        parser_func=pd.read_json,
+
+    return _read_text_format(
+        read_format="json",
         path=path,
         path_suffix=path_suffix,
         path_ignore_suffix=path_ignore_suffix,
+        version_id=version_id,
+        ignore_empty=ignore_empty,
         use_threads=use_threads,
-        boto3_session=boto3_session,
+        s3_client=s3_client,
         s3_additional_kwargs=s3_additional_kwargs,
         chunksize=chunksize,
         dataset=dataset,
@@ -546,5 +646,6 @@ def read_json(
         last_modified_begin=last_modified_begin,
         last_modified_end=last_modified_end,
         ignore_index=ignore_index,
+        ray_args=ray_args,
         **pandas_kwargs,
     )
